using System.Diagnostics;
using GraphicsManager.Enums;
using SixLabors.Fonts;

namespace GraphicsManager.Objects.Core;

public class FontFamily
{
    private static uint index = 0;
    public event Func<Task>? ReloadUI; 
    private FontSize fs = FontSize.Thin;
    private uint ph = 12;
    private bool i;
    internal Dictionary<FontSize, Tuple<string, Font?>> InternalFonts = new();
    internal Dictionary<FontSize, Tuple<string, Font?>> InternalFontsi = new();
    internal Font Font = null!;

    
    
    public static Task<FontFamily> LoadFontFamily()
    {
        FontFamily f = new();
        foreach (string font in Tools.GetFontList($"fc-list ':'family='{Tools.GetFamilyList("fc-list ':' family")[index]}' file"))
        {
            string fd = font.ToLower();
            string[] fda = fd.Split('.');
            fd = fda[fda.Length - 2];

            void tryadd(FontSize s, bool i = false)
            {
                if (i)
                {
                    if (!f.InternalFontsi.ContainsKey(s))
                        f.InternalFontsi.Add(s, new(font, null));
                }
                else
                {
                    if (!f.InternalFonts.ContainsKey(s))
                        f.InternalFonts.Add(s, new(font, null));
                }
            }
            
            if (fd.EndsWith("italic"))
            {
                if (fd.EndsWith("thinitalic")) tryadd(FontSize.Thin, true);
                else if (fd.EndsWith("extralightitalic")) tryadd(FontSize.ExtraLight, true);
                else if (fd.EndsWith("lightitalic")) tryadd(FontSize.Light, true);
                else if (fd.EndsWith("regularitalic")) tryadd(FontSize.Regular, true);
                else if (fd.EndsWith("mediumitalic")) tryadd(FontSize.Medium, true);
                else if (fd.EndsWith("semibolditalic")) tryadd(FontSize.SemiBold, true);
                else if (fd.EndsWith("bolditalic")) tryadd(FontSize.Bold, true);
                else if (fd.EndsWith("extrabolditalic")) tryadd(FontSize.ExtraBold, true);
                else if (fd.EndsWith("blackitalic")) tryadd(FontSize.Black, true);
                else tryadd(FontSize.Regular, true);
            }
            else
            {
                if (fd.EndsWith("thin")) tryadd(FontSize.Thin);
                else if (fd.EndsWith("extralight")) tryadd(FontSize.ExtraLight);
                else if (fd.EndsWith("light")) tryadd(FontSize.Light);
                else if (fd.EndsWith("regular")) tryadd(FontSize.Regular);
                else if (fd.EndsWith("medium")) tryadd(FontSize.Medium);
                else if (fd.EndsWith("semibold")) tryadd(FontSize.SemiBold);
                else if (fd.EndsWith("bold")) tryadd(FontSize.Bold);
                else if (fd.EndsWith("extrabold")) tryadd(FontSize.ExtraBold);
                else if (fd.EndsWith("black")) tryadd(FontSize.Black);
                else tryadd(FontSize.Regular);
            }
        }

        index++;
        return Task.FromResult(f);
    }


    public static Task<FontFamily> LoadFontFamily(string Family)
    {
        FontFamily f = new();
        foreach (string font in Tools.GetFontList($"fc-list ':'family='{Family}' file"))
        {
            string fd = font.ToLower();
            string[] fda = fd.Split('.');
            fd = fda[fda.Length - 2];

            void tryadd(FontSize s, bool i = false)
            {
                if (i)
                {
                    if (!f.InternalFontsi.ContainsKey(s))
                        f.InternalFontsi.Add(s, new(font, null));
                }
                else
                {
                    if (!f.InternalFonts.ContainsKey(s))
                        f.InternalFonts.Add(s, new(font, null));
                }
            }
            
            if (fd.EndsWith("italic"))
            {
                if (fd.EndsWith("thinitalic")) tryadd(FontSize.Thin, true);
                if (fd.EndsWith("extralightitalic")) tryadd(FontSize.ExtraLight, true);
                if (fd.EndsWith("lightitalic")) tryadd(FontSize.Light, true);
                if (fd.EndsWith("regularitalic")) tryadd(FontSize.Regular, true);
                if (fd.EndsWith("mediumitalic")) tryadd(FontSize.Medium, true);
                if (fd.EndsWith("semibolditalic")) tryadd(FontSize.SemiBold, true);
                if (fd.EndsWith("bolditalic")) tryadd(FontSize.Bold, true);
                if (fd.EndsWith("extrabolditalic")) tryadd(FontSize.ExtraBold, true);
                if (fd.EndsWith("blackitalic")) tryadd(FontSize.Black, true);
            }
            else
            {
                if (fd.EndsWith("thin")) tryadd(FontSize.Thin);
                if (fd.EndsWith("extralight")) tryadd(FontSize.ExtraLight);
                if (fd.EndsWith("light")) tryadd(FontSize.Light);
                if (fd.EndsWith("regular")) tryadd(FontSize.Regular);
                if (fd.EndsWith("medium")) tryadd(FontSize.Medium);
                if (fd.EndsWith("semibold")) tryadd(FontSize.SemiBold);
                if (fd.EndsWith("bold")) tryadd(FontSize.Bold);
                if (fd.EndsWith("extrabold")) tryadd(FontSize.ExtraBold);
                if (fd.EndsWith("black")) tryadd(FontSize.Black);
            }
        }
        return Task.FromResult(f);
    }
    
    public static async Task<FontFamily> LoadFontFamily(string Dir, string Family)
    {
        FontFamily f = new();
        return f;
    }
    
    private FontFamily()
    {
    }
}