using GraphicsManager.Enums;

namespace GraphicsManager.Objects.Core;

public class FontInteraction
{
    public FontFamily Family { get; private set; }
    public Font CurrentFont { get; private set; }
    private FontSize fs = FontSize.Thin;
    private uint ph = 12;
    private bool i;
    private uint epx = 0;
    public event Func<Task>? ReloadUI;
    
    public FontInteraction Clone()
    {
        return (FontInteraction)this.MemberwiseClone();
    }

    public static FontInteraction Load(FontFamily family)
    {
        FontInteraction c = new(family);
        c.FontSize = FontSize.Regular;
        return c;
    }
    private FontInteraction(FontFamily Family)
    {
        this.Family = Family;
    }
    
    public uint PixelHeight
    {
        get => ph;
        set
        {
            ph = value;
            if (ReloadUI is not null) ReloadUI.Invoke();
        }
    }
    
    public uint ExtraLinePixels
    {
        get => epx;
        set
        {
            epx = value;
            if (ReloadUI is not null) ReloadUI.Invoke();
        }
    }
    
    public bool Italic
    {
        get => i;
        set
        {
            if (i != value)
            {
                if (value)
                {
                    if (Family.InternalFontsi.TryGetValue(FontSize, out Tuple<string, Font?>? f))
                    {
                        i = value;
                        if (f.Item2 is null)
                        {
                            switch (Italic)
                            {
                                case true:
                                    Family.InternalFontsi[fs] = new(f.Item1, Font.MakeFontFromFile(Family.InternalFontsi[fs].Item1));
                                    break;
                                case false:
                                    Family.InternalFonts[fs] = new(f.Item1, Font.MakeFontFromFile(Family.InternalFonts[fs].Item1));
                                    break;
                            }
                        }

                        CurrentFont = (Family.InternalFontsi[fs].Item2!);
                        if (ReloadUI is not null) ReloadUI.Invoke();
                    }
                }
                else
                {
                    if (Family.InternalFonts.TryGetValue(FontSize, out Tuple<string, Font?>? f))
                    {
                        i = value;
                        if (f.Item2 is null)
                        {
                            switch (Italic)
                            {
                                case true:
                                    Family.InternalFontsi[fs] = new(f.Item1, Font.MakeFontFromFile(Family.InternalFontsi[fs].Item1));
                                    break;
                                case false:
                                    Family.InternalFonts[fs] = new(f.Item1, Font.MakeFontFromFile(Family.InternalFonts[fs].Item1));
                                    break;
                            }
                        }
                        CurrentFont = Family.InternalFonts[fs].Item2!;
                        if (ReloadUI is not null) ReloadUI.Invoke();
                    }
                }
                //Font.PixelHeight = this.PixelHeight;
            }
        }
    }
    
    private static Tuple<string, Font?> gcl(Dictionary<FontSize, Tuple<string, Font?>> thisList, FontSize thisValue, out FontSize fs)
    {
        Dictionary<FontSize, Tuple<string, Font?>>.KeyCollection keys = thisList.Keys;
        FontSize nearest = thisValue -
                           keys.Where(k => k <= thisValue)
                               .Min(k => thisValue - k);
        fs = nearest;
        return thisList[nearest];
    }
    
    public FontSize FontSize
    {
        get => fs;
        set
        {
            Tuple<string, Font?> f = Italic switch
            {
                true => gcl(Family.InternalFontsi, value, out fs),
                false => gcl(Family.InternalFonts, value, out fs),
            };
            if (f.Item2 is null)
            {
                switch (Italic)
                {
                    case true:
                        Family.InternalFontsi[fs] = new(f.Item1, Font.MakeFontFromFile(Family.InternalFontsi[fs].Item1));
                        break;
                    case false:
                        Family.InternalFonts[fs] = new(f.Item1, Font.MakeFontFromFile(Family.InternalFonts[fs].Item1));
                        break;
                }
            }

            CurrentFont = Italic switch
            {
                true => Family.InternalFontsi[fs].Item2!,
                false => Family.InternalFonts[fs].Item2!,
            };
            //Font.PixelHeight = this.PixelHeight;
            
            if (ReloadUI is not null) ReloadUI.Invoke();
        }
    }
}